# Generated by Django 4.0.1 on 2022-02-08 21:44

from django.db import migrations

from juntagrico.util.temporal import weekdays


def set_location_sort_order(apps, schema_editor):
    location = apps.get_model('juntagrico', 'Location')
    for idx, item in enumerate(location.objects.all()):
        item.sort_order = idx + 1
        item.save()


def make_name_product_unique(apps, schema_editor):
    SubscriptionSize = apps.get_model('juntagrico', 'SubscriptionSize')
    taken = set()
    for size in SubscriptionSize.objects.all():
        while (size.name, size.product) in taken:
            size.name += str(size.id)
        size.save()
        taken.add((size.name, size.product))


def set_tour(apps, schema_editor):
    depot = apps.get_model('juntagrico', 'Depot')
    tour = apps.get_model('juntagrico', 'Tour')
    delivery = apps.get_model('juntagrico', 'Delivery')
    for d in depot.objects.all():
        if 8 > d.weekday > 0:
            d.tour = tour.objects.get_or_create(name=weekdays[d.weekday])[0]
            d.save()
    for d in delivery.objects.all():
        d.tour = tour.objects.get_or_create(name=weekdays[d.delivery_date.weekday() + 1])[0]
        d.save()


class Migration(migrations.Migration):

    dependencies = [
        ('juntagrico', '0038_pre_1_6'),
    ]

    operations = [
        migrations.RunPython(set_location_sort_order),
        migrations.RunPython(make_name_product_unique),
        migrations.RunPython(set_tour),
    ]
